module Home.Helpers exposing (..)

import Blog.Helpers exposing (..)
import Config.Colour as T exposing (..)
import Config.Format as O exposing (..)
import Config.Identity as I exposing (..)
import Config.Response exposing (..)
import Config.Viewport exposing (..)
import Effect exposing (Effect)
import Element as E exposing (..)
import Element.Background as B exposing (..)
import Element.Border as D exposing (..)
import Element.Font as F
import Html.Attributes as H exposing (style)
import Layouts
import Page exposing (Page)
import Route exposing (Route)
import Shared exposing (..)
import View exposing (View)


homePage : { src : String, description : String } -> Element msg
homePage image =
    column
        [ centerX
        , centerY
        , spacing 20
        ]
        [ row [ centerX ]
            [ E.image [ E.width fill, height <| px 100 ]
                image
            ]
        , column
            [ paddingEach
                { top = 15
                , bottom = 15
                , left = 20
                , right = 20
                }
            , B.color colourTheme.backgroundDarkGrey
            , rounded 10
            , E.width fill
            , spacing 8
            ]
            [ paragraph (paragraphFormat ++ [ centerX ]) [ text "upRootNutrition is an open source project, created by Nick Hiebert, designed to elevate the quality of nutrition science communication in online discourse. By applying more rigorous systems of reasoning, such as formal logic, upRootNutrition aims to cut through the misinformation and sophistry that are endemic on social media." ]
            ]
        ]


desktopHomePageImage : { src : String, description : String }
desktopHomePageImage =
    { src = "assets/logo_extended.png"
    , description = ""
    }


desktopHomePage =
    homePage desktopHomePageImage


mobileLargeHomePageImage : { src : String, description : String }
mobileLargeHomePageImage =
    { src = "assets/logo.png"
    , description = ""
    }


mobileLargeHomePage =
    homePage mobileLargeHomePageImage


mobileSmallHomePageImage : { src : String, description : String }
mobileSmallHomePageImage =
    { src = "assets/logo_lambda.png"
    , description = ""
    }


mobileSmallHomePage =
    homePage mobileSmallHomePageImage
